@@ -13,6 +13,30 @@ use which::which;
 fn main() {
   println!("cargo:rerun-if-changed=src/binding.cc");
 
+  // These are all the environment variables that we check. This is
+  // probably more than what is needed, but missing an important
+  // variable can lead to broken links when switching rusty_v8
+  // versions.
+  let envs = vec![
+    "CCACHE",
+    "CLANG_BASE_PATH",
+    "DENO_TRYBUILD",
+    "DOCS_RS",
+    "GN",
+    "GN_ARGS",
+    "HOST",
+    "NINJA",
+    "OUT_DIR",
+    "RUSTY_V8_ARCHIVE",
+    "RUSTY_V8_MIRROR",
+    "SCCACHE",
+    "V8_FORCE_DEBUG",
+    "V8_FROM_SOURCE",
+  ];
+  for env in envs {
+    println!("cargo:rerun-if-env-changed={}", env);
+  }
+
   // Detect if trybuild tests are being compiled.
   let is_trybuild = env::var_os("DENO_TRYBUILD").is_some();
 
@@ -201,19 +225,32 @@ fn download_ninja_gn_binaries() {
 }
 
 fn static_lib_url() -> String {
+  if let Ok(custom_archive) = env::var("RUSTY_V8_ARCHIVE") {
+    return custom_archive;
+  }
   let default_base = "https://github.com/denoland/rusty_v8/releases/download";
   let base =
     env::var("RUSTY_V8_MIRROR").unwrap_or_else(|_| default_base.into());
   let version = env::var("CARGO_PKG_VERSION").unwrap();
   let target = env::var("TARGET").unwrap();
+
+  // Note: we always use the release build on windows.
   if cfg!(target_os = "windows") {
-    // Note: we always use the release build on windows.
-    format!("{}/v{}/rusty_v8_release_{}.lib", base, version, target)
-  } else {
-    let profile = env::var("PROFILE").unwrap();
-    assert!(profile == "release" || profile == "debug");
-    format!("{}/v{}/librusty_v8_{}_{}.a", base, version, profile, target)
+    return format!("{}/v{}/rusty_v8_release_{}.lib", base, version, target);
   }
+  // Use v8 in release mode unless $V8_FORCE_DEBUG=true
+  let profile = match env_bool("V8_FORCE_DEBUG") {
+    true => "debug",
+    _ => "release",
+  };
+  format!("{}/v{}/librusty_v8_{}_{}.a", base, version, profile, target)
+}
+
+fn env_bool(key: &str) -> bool {
+  matches!(
+    env::var(key).unwrap_or_default().as_str(),
+    "true" | "1" | "yes"
+  )
 }
 
 fn static_lib_name() -> &'static str {
@@ -300,11 +337,10 @@ fn download_static_lib_binaries() {
 
   let filename = static_lib_path();
   if filename.exists() {
-    println!("static lib already exists {}", filename.display());
-    println!("To re-download this file, it must be manually deleted.");
-  } else {
-    download_file(url, filename);
+    println!("Deleting old static lib {}", filename.display());
+    std::fs::remove_file(&filename).unwrap();
   }
+  download_file(url, filename);
 }
 
 fn print_link_flags() {
