@@ -131,8 +131,11 @@ namespace internal {
   /* Time for lazily compiling Wasm functions. */                             \
   HT(wasm_lazy_compile_time, V8.WasmLazyCompileTimeMicroSeconds, 100000000,   \
      MICROSECOND)                                                             \
-  /* Total time to decompress snapshots. */                                   \
-  HT(snapshot_decompress, V8.SnapshotDecompress, 10000000, MICROSECOND)
+  /* Total time to decompress isolate snapshot. */                            \
+  HT(snapshot_decompress, V8.SnapshotDecompress, 10000000, MICROSECOND)       \
+  /* Time to decompress context snapshot. */                                  \
+  HT(context_snapshot_decompress, V8.ContextSnapshotDecompress, 10000000,     \
+     MICROSECOND)
 
 #define NESTED_TIMED_HISTOGRAM_LIST_SLOW(HT)                               \
   /* Total V8 time (including JS and runtime calls, exluding callbacks) */ \
