@@ -31,8 +31,7 @@ Handle<ScriptContextTable> ScriptContextTable::Extend(
     result = table;
   }
   DCHECK(script_context->IsScriptContext());
-  result->set(used + kFirstContextSlotIndex, *script_context);
-
+  result->set(used + kFirstContextSlotIndex, *script_context, kReleaseStore);
   result->set_used(used + 1, kReleaseStore);
   return result;
 }
@@ -66,7 +65,7 @@ bool ScriptContextTable::Lookup(Isolate* isolate, ScriptContextTable table,
   return false;
 }
 
-bool Context::is_declaration_context() {
+bool Context::is_declaration_context() const {
   if (IsFunctionContext() || IsNativeContext() || IsScriptContext() ||
       IsModuleContext()) {
     return true;
@@ -78,7 +77,7 @@ bool Context::is_declaration_context() {
   return scope_info().is_declaration_scope();
 }
 
-Context Context::declaration_context() {
+Context Context::declaration_context() const {
   Context current = *this;
   while (!current.is_declaration_context()) {
     current = current.previous();
@@ -86,7 +85,7 @@ Context Context::declaration_context() {
   return current;
 }
 
-Context Context::closure_context() {
+Context Context::closure_context() const {
   Context current = *this;
   while (!current.IsFunctionContext() && !current.IsScriptContext() &&
          !current.IsModuleContext() && !current.IsNativeContext() &&
@@ -96,7 +95,7 @@ Context Context::closure_context() {
   return current;
 }
 
-JSObject Context::extension_object() {
+JSObject Context::extension_object() const {
   DCHECK(IsNativeContext() || IsFunctionContext() || IsBlockContext() ||
          IsEvalContext() || IsCatchContext());
   HeapObject object = extension();
@@ -106,17 +105,13 @@ JSObject Context::extension_object() {
   return JSObject::cast(object);
 }
 
-JSReceiver Context::extension_receiver() {
+JSReceiver Context::extension_receiver() const {
   DCHECK(IsNativeContext() || IsWithContext() || IsEvalContext() ||
          IsFunctionContext() || IsBlockContext());
   return IsWithContext() ? JSReceiver::cast(extension()) : extension_object();
 }
 
-ScopeInfo Context::scope_info() {
-  return ScopeInfo::cast(get(SCOPE_INFO_INDEX));
-}
-
-SourceTextModule Context::module() {
+SourceTextModule Context::module() const {
   Context current = *this;
   while (!current.IsModuleContext()) {
     current = current.previous();
@@ -124,11 +119,11 @@ SourceTextModule Context::module() {
   return SourceTextModule::cast(current.extension());
 }
 
-JSGlobalObject Context::global_object() {
+JSGlobalObject Context::global_object() const {
   return JSGlobalObject::cast(native_context().extension());
 }
 
-Context Context::script_context() {
+Context Context::script_context() const {
   Context current = *this;
   while (!current.IsScriptContext()) {
     current = current.previous();
@@ -136,7 +131,7 @@ Context Context::script_context() {
   return current;
 }
 
-JSGlobalProxy Context::global_proxy() {
+JSGlobalProxy Context::global_proxy() const {
   return native_context().global_proxy_object();
 }
 
@@ -447,6 +442,47 @@ int Context::IntrinsicIndexForName(const unsigned char* unsigned_string,
 
 #undef COMPARE_NAME
 
+#ifdef VERIFY_HEAP
+namespace {
+// TODO(v8:12298): Fix js-context-specialization cctests to set up full
+// native contexts instead of using dummy internalized strings as
+// extensions.
+bool IsContexExtensionTestObject(HeapObject extension) {
+  return extension.IsInternalizedString() &&
+         String::cast(extension).length() == 1;
+}
+}  // namespace
+
+void Context::VerifyExtensionSlot(HeapObject extension) {
+  CHECK(scope_info().HasContextExtensionSlot());
+  // Early exit for potentially uninitialized contexfts.
+  if (extension.IsUndefined()) return;
+  if (extension.IsJSContextExtensionObject()) {
+    CHECK((IsBlockContext() && scope_info().is_declaration_scope()) ||
+          IsFunctionContext());
+  } else if (IsModuleContext()) {
+    CHECK(extension.IsSourceTextModule());
+  } else if (IsDebugEvaluateContext() || IsWithContext()) {
+    CHECK(extension.IsJSReceiver() ||
+          (IsWithContext() && IsContexExtensionTestObject(extension)));
+  } else if (IsNativeContext()) {
+    CHECK(extension.IsJSGlobalObject() ||
+          IsContexExtensionTestObject(extension));
+  } else if (IsScriptContext()) {
+    // Host-defined options can be stored on the context for classic scripts.
+    CHECK(extension.IsFixedArray());
+  }
+}
+#endif  // VERIFY_HEAP
+
+void Context::set_extension(HeapObject object, WriteBarrierMode mode) {
+  DCHECK(scope_info().HasContextExtensionSlot());
+#ifdef VERIFY_HEAP
+  VerifyExtensionSlot(object);
+#endif
+  set(EXTENSION_INDEX, object, mode);
+}
+
 #ifdef DEBUG
 
 bool Context::IsBootstrappingOrValidParentContext(Object object,
@@ -525,7 +561,15 @@ void NativeContext::RunPromiseHook(PromiseHookType type,
 
   Handle<Object> receiver = isolate->global_proxy();
 
-  if (Execution::Call(isolate, hook, receiver, argc, argv).is_null()) {
+  StackLimitCheck check(isolate);
+  bool failed = false;
+  if (check.HasOverflowed()) {
+    isolate->StackOverflow();
+    failed = true;
+  } else {
+    failed = Execution::Call(isolate, hook, receiver, argc, argv).is_null();
+  }
+  if (failed) {
     DCHECK(isolate->has_pending_exception());
     Handle<Object> exception(isolate->pending_exception(), isolate);
 
